/*
 * Copyright (C) 2007 Onno Kortmann <onno@gmx.net>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *  
 */

/* Unfinished verilog-side glue code to the simulavrxx AVR devices. */

module avr_pin(core, out, in);
   parameter name="UNSPECIFIED";
   input core;
   output out;
   reg 	  out;
   // FIXME: inout for one port?
   input  in;
   integer val;

   initial begin
      out<= 1'bz;
   end
   
   function a2v;
      input apin;
      if (apin!=0)
	a2v=1;
      else a2v=0;
   endfunction // a2v

   function v2a;
      input vpin;
      if (vpin!=0)
	v2a=1;
      else v2a=0;
   endfunction // v2a
   
   always @(posedge core.clk) begin
      $avr_get_pin(core.handle, name, val);
      out<=a2v(val);
      $avr_set_pin(core.handle, name, v2a(in));
   end
   
endmodule // avr_pin

module avr_clock(clk);
   output clk;
   reg 	  clk;
   parameter FREQ=4_000_000;
   initial begin
      clk<=0;
   end
   
   always @(clk) begin
      #(1_000_000_000/FREQ/2) clk<=~clk; //125000 -> 4MHz clock
   end   
endmodule // avr_clock
    
module AVRCORE(clk);
   parameter progfile="UNSPECIFIED";
   parameter name="UNSPECIFIED";
   input     clk;

   integer   handle;
   
   initial begin
      $display("Creating an AVR device.");
      $avr_create(handle, name, progfile);
      $avr_reset(handle);
   end

   always @(posedge clk) begin
      $avr_tick(handle);
   end
   
endmodule // AVRCORE

module AT90S4433(clk, portb, pinb);
   parameter progfile="UNSPECIFIED";
   input     clk;
   output [7:0] portb;
   input [7:0] 	pinb;

   integer 	handle;

   defparam 	core.progfile=progfile;
   defparam 	core.name="AT90S4433";
   AVRCORE core(clk);
   
   avr_pin #("B0") pb0(core, portb[0], pinb[0]);
   avr_pin #("B1") pb1(core, portb[1], pinb[1]);
   avr_pin #("B2") pb2(core, portb[2], pinb[2]);
   avr_pin #("B3") pb3(core, portb[3], pinb[3]);
   avr_pin #("B4") pb4(core, portb[4], pinb[4]);
   avr_pin #("B5") pb5(core, portb[5], pinb[5]);
   avr_pin #("B6") pb6(core, portb[6], pinb[6]);
   avr_pin #("B7") pb7(core, portb[7], pinb[7]);
   
endmodule

